from enum import Enum
from typing import Union

import numpy as np

from rumorz_data.constants import convert_entity_type_format


class ScreenerValues(Enum):
    ABS = "abs"
    CHG = "chg"


class Lookback(Enum):
    ONE_HOUR = "1H"
    SIX_HOURS = "6H"
    TWELVE_HOURS = "12H"
    ONE_DAY = "1D"
    ONE_WEEK = "7D"
    ONE_MONTH = "30D"
    THREE_MONTHS = "90D"
    ONE_YEAR = "365D"


class EntityType(Enum):
    FINANCIAL_ASSET = "financial_asset"
    COMPANY = "company"
    ORGANIZATION = "organization"
    PERSON = "person"
    PLACE = "place"


class AssetClass(Enum):
    CRYPTO = "crypto"


class SearchMethod(Enum):
    EXACT = "exact"
    CONTAINS = "contains"
    KEYWORD = "keyword"


class NodeMetrics(Enum):
    SENTIMENT = 'sentiment'
    MENTIONS = 'mentions'
    EXCITEMENT = 'excitement'
    OPTIMISM = 'optimism'
    PESSIMISM = 'pessimism'
    FEAR = 'fear'
    UNCERTAINTY = 'uncertainty'
    SURPRISE = 'surprise'


class Entity:
    def __init__(self,
                 type: Union[EntityType, str],
                 name,
                 symbol=None):
        self.type = type
        self.name = name
        if isinstance(symbol, str):
            self.symbol = symbol
        else:
            self.symbol = None


    def to_dict(self,
                format=None):
        return {
            "name": self.name,
            "type": self.type if format is None else convert_entity_type_format(getattr(self.type, 'value', self.type),
                                                                                format),
            "symbol": self.symbol
        }



import os
import datetime as dt
import unittest
from rumorz.client import RumorzClient, RumorzAPIException
from rumorz.enums import Lookback, ScreenerValues, EntityType

rumorz = RumorzClient(api_key=os.environ['RUMORZ_API_KEY'],
                      api_url='http://0.0.0.0:80')


class TestRumorz(unittest.TestCase):

    def test_valid_screener_chg_lookback_request(self):
        person_screener = rumorz.graph.get_screener(
            lookback=Lookback.THREE_MONTHS,
            entity_type=EntityType.PERSON
        )
        self.assertTrue(len(person_screener['data']) > 0)

    def test_valid_screener_abs_timestamp_request(self):
        to_timestamp, from_timestamp = dt.datetime.utcnow().isoformat(), (dt.datetime.utcnow() - dt.timedelta(days=14)).isoformat()
        person_screener = rumorz.graph.get_screener(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            entity_type=EntityType.PERSON
        )
        self.assertTrue(len(person_screener) > 0)

    def test_invalid_screener_abs_timestamp_request(self):
        from_timestamp, to_timestamp = (dt.datetime.utcnow() - dt.timedelta(days=7)).isoformat(), dt.datetime.utcnow().isoformat()
        try:
            person_screener = rumorz.graph.get_screener(
                from_timestamp=5.2,
                to_timestamp=to_timestamp,
                entity_type=EntityType.PERSON
            )
            print(person_screener)
        except Exception as e:
            self.assertTrue(isinstance(e, RumorzAPIException))


    def test_posts(self):
        posts = rumorz.graph.get_posts(
            type=EntityType.FINANCIAL_ASSET,
            name="Bitcoin",
            symbol="BTC",
            lookback=Lookback.ONE_WEEK,
            page=2,
        )
        self.assertTrue(len(posts) > 0)


    def test_time_series(self):
        timeseries = rumorz.graph.get_time_series(
            type=EntityType.FINANCIAL_ASSET,
            name="Bitcoin",
            symbol="BTC",
            lookback=Lookback.ONE_WEEK,
            page=1,
        )
        self.assertTrue(len(timeseries) > 0)


from enum import Enum
from typing import List, Union
from urllib.parse import urlencode

import requests

from rumorz.enums import SearchMethod, EntityType, AssetClass, NodeMetrics, Lookback, ScreenerValues


class RumorzAPIException(Exception):
    pass


class RumorzClient:
    def __init__(self,
                 api_key,
                 api_url='https://prod-backend-rumorz-l2cw8.ondigitalocean.app'):
        self.api_url = api_url
        self.headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json',
            'X-API-Key': api_key
        }
        self._graph = self.Graph(self)
        self._agent = self.Agent(self)

    def _format_data(self, data):
        for key, value in data.items():
            if isinstance(value, Enum):
                data[key] = value.value

        todrop = []
        for key, value in data.items():
            if value is None:
                todrop.append(key)
        for key in todrop: data.pop(key)

    def post(self, endpoint, data):
        url = f"{self.api_url}/{endpoint}"
        self._format_data(data)
        response = requests.post(url, headers=self.headers, json=data)
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise RumorzAPIException(response.json()['detail'])
        return response.json()

    def get(self, endpoint, data):
        url = f"{self.api_url}/{endpoint}"
        self._format_data(data)
        query_params_string = urlencode(data, doseq=True)
        # query_params_string = '%7B%27entities%27%3A%20%5B%7B%27type%27%3A%20%27financial_asset%27%2C%20%27name%27%3A%20%27Bitcoin%27%2C%20%27symbol%27%3A%20%27BTC%27%7D%5D%2C%20%27lookback%27%3A%20%277D%27%2C%20%27page%27%3A%201%2C%20%27limit%27%3A%2010%7D%0A'
        # query_params_string = "&".join([f"{k}={v}" for k, v in data.items() if v is not None])
        query_url = f"{url}?{query_params_string}"
        response = requests.get(query_url, headers=self.headers)
        response.text
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise RumorzAPIException(response.json()['detail'])
        return response.json()

    @property
    def graph(self):
        return self._graph

    @property
    def agent(self):
        return self._agent

    class Graph:
        def __init__(self, api):
            self.api = api

        def get_screener(self,
                         lookback: Union[str, Lookback] = None,
                         from_timestamp: str = None,
                         to_timestamp: str = None,
                         entity_type: Union[str, EntityType] = None):
            params = {
                "from_timestamp": from_timestamp,
                "to_timestamp": to_timestamp,
                "lookback": lookback,
                "entity_type": entity_type
            }
            return self.api.get('graph/screener', params)

        def get_posts(self,
                      type: Union[str, EntityType],
                      name: str,
                      symbol: str = None,
                      lookback: Union[str, Lookback] = None,
                      from_timestamp: str = None,
                      to_timestamp: str = None,
                      page: int = 1,
                      limit: int = 10):
            params = {
                "type": type,
                "name": name,
                "symbol": symbol,
                "lookback": lookback,
                "from_timestamp": from_timestamp,
                "to_timestamp": to_timestamp,
                "page": page,
                "limit": limit
            }
            return self.api.get('graph/posts', params)

        def get_time_series(self,
                            type: Union[str, EntityType],
                            name: str,
                            symbol: str = None,
                            lookback: Union[str, Lookback] = None,
                            from_timestamp: str = None,
                            to_timestamp: str = None,
                            page: int = 1,
                            limit: int = 10):
            params = {
                "type": type,
                "name": name,
                "symbol": symbol,
                "lookback": lookback,
                "from_timestamp": from_timestamp,
                "to_timestamp": to_timestamp,
                "page": page,
                "limit": limit
            }
            return self.api.get('graph/time-series', params)

    class Agent:
        def __init__(self, api):
            self.api = api
